@@ -4,14 +4,13 @@ package tools
 
 import (
 	_ "knative.dev/hack"
+	_ "knative.dev/pkg/hack"
 
 	// For chaos testing the leaderelection stuff.
 	_ "knative.dev/pkg/leaderelection/chaosduck"
 
 	_ "knative.dev/networking/test/conformance/ingress"
-	_ "knative.dev/networking/test/test_images/flaky"
 	_ "knative.dev/networking/test/test_images/grpc-ping"
-	_ "knative.dev/networking/test/test_images/helloworld"
 	_ "knative.dev/networking/test/test_images/httpproxy"
 	_ "knative.dev/networking/test/test_images/runtime"
 	_ "knative.dev/networking/test/test_images/timeout"
