@@ -20,17 +20,31 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"os"
 	"sync"
 
 	v2 "github.com/envoyproxy/go-control-plane/envoy/api/v2"
 	route "github.com/envoyproxy/go-control-plane/envoy/api/v2/route"
-	"github.com/envoyproxy/go-control-plane/pkg/cache"
+	httpconnmanagerv2 "github.com/envoyproxy/go-control-plane/envoy/config/filter/network/http_connection_manager/v2"
+	cachetypes "github.com/envoyproxy/go-control-plane/pkg/cache/types"
+	cache "github.com/envoyproxy/go-control-plane/pkg/cache/v2"
 	"github.com/golang/protobuf/ptypes/wrappers"
 	"github.com/google/uuid"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/sets"
 	kubeclient "k8s.io/client-go/kubernetes"
 	"knative.dev/net-kourier/pkg/config"
+	envoy "knative.dev/net-kourier/pkg/envoy/api"
+)
+
+const (
+	envCertsSecretNamespace = "CERTS_SECRET_NAMESPACE"
+	envCertsSecretName      = "CERTS_SECRET_NAME"
+	certFieldInSecret       = "tls.crt"
+	keyFieldInSecret        = "tls.key"
+	externalRouteConfigName = "external_services"
+	internalRouteConfigName = "internal_services"
 )
 
 var ErrDomainConflict = errors.New("ingress has a conflicting domain with another ingress")
@@ -40,8 +54,6 @@ type Caches struct {
 	translatedIngresses map[types.NamespacedName]*translatedIngress
 	clusters            *ClustersCache
 	domainsInUse        sets.String
-	routeConfig         []*v2.RouteConfiguration
-	listeners           []*v2.Listener
 	statusVirtualHost   *route.VirtualHost
 
 	kubeClient kubeclient.Interface
@@ -59,10 +71,6 @@ func NewCaches(ctx context.Context, kubernetesClient kubeclient.Interface, extAu
 	if extAuthz {
 		c.clusters.set(config.ExternalAuthz.Cluster, "__extAuthZCluster", "_internal")
 	}
-
-	if err := c.setListeners(ctx); err != nil {
-		return nil, err
-	}
 	return c, nil
 }
 
@@ -73,12 +81,7 @@ func (caches *Caches) UpdateIngress(ctx context.Context, ingressTranslation *tra
 	defer caches.mu.Unlock()
 
 	caches.deleteTranslatedIngress(ingressTranslation.name.Name, ingressTranslation.name.Namespace)
-
-	if err := caches.addTranslatedIngress(ingressTranslation); err != nil {
-		return err
-	}
-
-	return caches.setListeners(ctx)
+	return caches.addTranslatedIngress(ingressTranslation)
 }
 
 func (caches *Caches) validateIngress(translatedIngress *translatedIngress) error {
@@ -120,7 +123,10 @@ func (caches *Caches) SetOnEvicted(f func(types.NamespacedName, interface{})) {
 	})
 }
 
-func (caches *Caches) setListeners(ctx context.Context) error {
+func (caches *Caches) ToEnvoySnapshot(ctx context.Context) (cache.Snapshot, error) {
+	caches.mu.Lock()
+	defer caches.mu.Unlock()
+
 	localVHosts := make([]*route.VirtualHost, 0, len(caches.translatedIngresses)+1)
 	externalVHosts := make([]*route.VirtualHost, 0, len(caches.translatedIngresses))
 	snis := sniMatches{}
@@ -136,42 +142,15 @@ func (caches *Caches) setListeners(ctx context.Context) error {
 	// Append the statusHost too.
 	localVHosts = append(localVHosts, caches.statusVirtualHost)
 
-	listeners, err := listenersFromVirtualHosts(
+	listeners, routes, err := generateListenersAndRouteConfigs(
 		ctx,
 		externalVHosts,
 		localVHosts,
 		snis.list(),
 		caches.kubeClient,
-		caches,
 	)
-
 	if err != nil {
-		return err
-	}
-
-	caches.listeners = listeners
-
-	return nil
-}
-
-func (caches *Caches) ToEnvoySnapshot() (cache.Snapshot, error) {
-	caches.mu.Lock()
-	defer caches.mu.Unlock()
-
-	// Instead of sending the Routes, we send the RouteConfigs.
-	routes := make([]cache.Resource, len(caches.routeConfig))
-	for i := range caches.routeConfig {
-		// Without this we can generate routes that point to non-existing clusters
-		// That causes some "no_cluster" errors in Envoy and the "TestUpdate"
-		// in the Knative serving test suite fails sometimes.
-		// Ref: https://github.com/knative/serving/blob/f6da03e5dfed78593c4f239c3c7d67c5d7c55267/test/conformance/ingress/update_test.go#L37
-		caches.routeConfig[i].ValidateClusters = &wrappers.BoolValue{Value: true}
-		routes[i] = caches.routeConfig[i]
-	}
-
-	listeners := make([]cache.Resource, len(caches.listeners))
-	for i := range caches.listeners {
-		listeners[i] = caches.listeners[i]
+		return cache.Snapshot{}, err
 	}
 
 	// Generate and append the internal kourier route for keeping track of the snapshot id deployed
@@ -183,11 +162,12 @@ func (caches *Caches) ToEnvoySnapshot() (cache.Snapshot, error) {
 
 	return cache.NewSnapshot(
 		snapshotVersion,
-		make([]cache.Resource, 0),
+		make([]cachetypes.Resource, 0),
 		caches.clusters.list(),
 		routes,
 		listeners,
-		make([]cache.Resource, 0),
+		make([]cachetypes.Resource, 0),
+		make([]cachetypes.Resource, 0),
 	), nil
 }
 
@@ -199,7 +179,7 @@ func (caches *Caches) DeleteIngressInfo(ctx context.Context, ingressName string,
 	defer caches.mu.Unlock()
 
 	caches.deleteTranslatedIngress(ingressName, ingressNamespace)
-	return caches.setListeners(ctx)
+	return nil
 }
 
 func (caches *Caches) deleteTranslatedIngress(ingressName, ingressNamespace string) {
@@ -231,3 +211,84 @@ func (caches *Caches) getNewSnapshotVersion() (string, error) {
 
 	return snapshotVersion.String(), nil
 }
+
+func generateListenersAndRouteConfigs(
+	ctx context.Context,
+	externalVirtualHosts []*route.VirtualHost,
+	clusterLocalVirtualHosts []*route.VirtualHost,
+	sniMatches []*envoy.SNIMatch,
+	kubeclient kubeclient.Interface) ([]cachetypes.Resource, []cachetypes.Resource, error) {
+
+	// First, we save the RouteConfigs with the proper name and all the virtualhosts etc. into the cache.
+	externalRouteConfig := envoy.NewRouteConfig(externalRouteConfigName, externalVirtualHosts)
+	internalRouteConfig := envoy.NewRouteConfig(internalRouteConfigName, clusterLocalVirtualHosts)
+
+	// Without this we can generate routes that point to non-existing clusters
+	// That causes some "no_cluster" errors in Envoy and the "TestUpdate"
+	// in the Knative serving test suite fails sometimes.
+	// Ref: https://github.com/knative/serving/blob/f6da03e5dfed78593c4f239c3c7d67c5d7c55267/test/conformance/ingress/update_test.go#L37
+	externalRouteConfig.ValidateClusters = &wrappers.BoolValue{Value: true}
+	internalRouteConfig.ValidateClusters = &wrappers.BoolValue{Value: true}
+
+	// Now we setup connection managers, that reference the routeconfigs via RDS.
+	externalManager := envoy.NewHTTPConnectionManager(externalRouteConfig.Name)
+	internalManager := envoy.NewHTTPConnectionManager(internalRouteConfig.Name)
+	externalHTTPEnvoyListener, err := envoy.NewHTTPListener(externalManager, config.HTTPPortExternal)
+	if err != nil {
+		return nil, nil, err
+	}
+	internalEnvoyListener, err := envoy.NewHTTPListener(internalManager, config.HTTPPortInternal)
+	if err != nil {
+		return nil, nil, err
+	}
+
+	listeners := []cachetypes.Resource{externalHTTPEnvoyListener, internalEnvoyListener}
+
+	// Configure TLS Listener. If there's at least one ingress that contains the
+	// TLS field, that takes precedence. If there is not, TLS will be configured
+	// using a single cert for all the services if the creds are given via ENV.
+	if len(sniMatches) > 0 {
+		externalHTTPSEnvoyListener, err := envoy.NewHTTPSListenerWithSNI(externalManager, config.HTTPSPortExternal, sniMatches)
+		if err != nil {
+			return nil, nil, err
+		}
+		listeners = append(listeners, externalHTTPSEnvoyListener)
+	} else if useHTTPSListenerWithOneCert() {
+		externalHTTPSEnvoyListener, err := newExternalEnvoyListenerWithOneCert(
+			ctx, externalManager, kubeclient,
+		)
+		if err != nil {
+			return nil, nil, err
+		}
+		listeners = append(listeners, externalHTTPSEnvoyListener)
+	}
+
+	return listeners, []cachetypes.Resource{externalRouteConfig, internalRouteConfig}, nil
+}
+
+// Returns true if we need to modify the HTTPS listener with just one cert
+// instead of one per ingress
+func useHTTPSListenerWithOneCert() bool {
+	return os.Getenv(envCertsSecretNamespace) != "" &&
+		os.Getenv(envCertsSecretName) != ""
+}
+
+func sslCreds(ctx context.Context, kubeClient kubeclient.Interface, secretNamespace string, secretName string) (certificateChain []byte, privateKey []byte, err error) {
+	secret, err := kubeClient.CoreV1().Secrets(secretNamespace).Get(ctx, secretName, metav1.GetOptions{})
+	if err != nil {
+		return nil, nil, err
+	}
+
+	return secret.Data[certFieldInSecret], secret.Data[keyFieldInSecret], nil
+}
+
+func newExternalEnvoyListenerWithOneCert(ctx context.Context, manager *httpconnmanagerv2.HttpConnectionManager, kubeClient kubeclient.Interface) (*v2.Listener, error) {
+	certificateChain, privateKey, err := sslCreds(
+		ctx, kubeClient, os.Getenv(envCertsSecretNamespace), os.Getenv(envCertsSecretName),
+	)
+	if err != nil {
+		return nil, err
+	}
+
+	return envoy.NewHTTPSListener(manager, config.HTTPSPortExternal, certificateChain, privateKey)
+}
