@@ -27,7 +27,7 @@ export DISABLE_MD_LINTING=1
 
 export GO111MODULE=on
 
-source $(dirname $0)/../vendor/knative.dev/test-infra/scripts/presubmit-tests.sh
+source $(dirname $0)/../vendor/knative.dev/hack/presubmit-tests.sh
 
 # TODO(mattmoor): integration tests
 
